/*!
 * @file BadgeV1.c
 *
 * @Use E-Paper LILYGO T5 V2.3.1 2.13" : 
 * https://fr.aliexpress.com/item/32869729970.html?spm=a2g0o.productlist.main.1.6dee235d3rFwu4&algo_pvid=495ef5d0-d2db-4b19-afb7-4247a290f077&algo_exp_id=495ef5d0-d2db-4b19-afb7-4247a290f077-0&pdp_ext_f=%7B%22sku_id%22%3A%2212000028124589187%22%7D&pdp_npi=2%40dis%21EUR%2113.54%2112.86%21%21%21%21%21%4021224e9b16749809768017597d0666%2112000028124589187%21sea&curPageLogUid=gxOJZbvjdhX6
 *
 * @section author Author
 *
 * Written by F5FDR - BADGE Electronique - Ve 2.0 - 01/02/2023
 * For the amateur radio hobby
 *
 * @section license License
 *
 * MIT license, all text above must be included in any redistribution.
 */

/*
 *  IDE Configuration : - Carte ESP32 Dev Module
 *                      - Upload speed 921600
 *                      - CPU Frequency : 240 MHz
 *                      - Flash frequency : 40 MHz
 *                      - Flash mode : DIO
 *                      - Flash size : 4 MB (32 Mb)
 *                      - Partition scheme : Default 4MB with spiffs (1.2 MB APP/1.5MB SPIFFS)
 *
 *  Board to be added : - ESP32                   1.0.6
 *
 *  Liens pour télécharger les fichiers dédiés à la carte d'éval ESP32: https://dl.espressif.com/dl/package_esp32_index.json
 *
 *  Library to use :    - GxEPD2                  1.5.0 et avec ses dépendances
 *                        - Adafruit_GFX_Library  1.11.5
 *                        - Adafruit busIO        1.14.1
 *                      - SD                      1.2.4
 *                      - ArduinoJSON             6.20.1  
 *
 *
 * FDR le 01/02/2023
 * 
 * Notes : Ce code est documenté afin de permettre aux OMs ne maîtrisant pas la programmation,
 *         de modifier eux mêmes les paramètres des textes pour les adapter à leur besoin.
 *         Une simple recompilation du code source et un téléchargement permet de mettre à jour l'affichage
 *         sans avoir besoin de compétences approfondies en programmation après avoir modifié les paramètres
 *         par défaut. De plus ces commentaires sont volontairement en Français car cette application est 
 *         principalement dédiée aux radioamateurs de l'exagone.
 *         
 *         Afin d'accéder à la modification des chaînes de caractères et permettre de configurer l'affichage
 *         avec les données utilisateur, il est aussi possible d'utiliser un fichier au format JSON qui 
 *         devra être stocké sur une carte µSD. A la mise sous tension, la carte SD sera lue et les données 
 *         du fichier remplaceront celles par défaut. Le fichier JSON doit être nommé "badge.json", il doit 
 *         être situé dans le "root" de la carte SD (pas dans un sous-répertoire). Il doit respecter la 
 *         syntaxe imposée par les spécificités JSON. Il est important de noter que la moindre erreur dans
 *         la syntaxe du fichier JSON fera échouer le chargement du fichier et les paramètre par défaut
 *         seront pris en compte.
 *         
 *         Une option de "DEBUG" via la liaison série (cordon USB de programmation) permet de suivre le
 *         démarrage du badge et permet d'identifier un éventuel défaut ou erreur dans le fichier JSON.
 *         Cette option de "DEBUG" est activée par défaut et elle peut être désactivée en manipulant la
 *         définition "__SERIAL_DEBUG__" (true / false).
 *         
 *         Les polices de caractères sont modifiables mais uniquement par modification du code. Il n'est pas 
 *         prévu, dans cette version, de passer la sélection du type de police de caractère via le fichier
 *         JSON.
 *         
 *         Le contenu du fichier JSON "badge.json" est le suivant:

{  
  "callsign" : {
    "mycall"  : "F5xxx",
    "x_pos"   : 70,
    "y_pos"   : 65,
    "enable"  : 1
  },
  "name" : {
    "myname"  : "Prenom",
    "x_pos"   : 105,
    "y_pos"   : 90,
    "enable"  : 1
  },
  "title" : {
    "title"   : "TITRE BADGE 20xx",
    "x_pos"   : 24,
    "y_pos"   : 18,
    "enable"  : 1
  },
  "ref" : {
    "refnum"  : "00000",
    "x_pos"   : 10,
    "y_pos"   : 116,
    "enable"  : 0
  },
  "logo" : {
    "x_pos"   : 1,
    "x_size"  : 50,
    "y_pos"   : 26,
    "y_size"  : 71,
    "reverse" : 0,
    "enable"  : 1
  },
  "qthloc" : {
    "qth"   : "JNnnCC",
    "x_pos"   : 170,
    "y_pos"   : 116,
    "enable"  : 1
  },
  "acronym" : {
    "text"    : "ABCD",
    "x_pos"   : 230,
    "y_pos"   : 38,
    "height"  : 19,
    "enable"  : 1
  },
  "freetext" : {
    "freetext"  : "myaddress@provider.com",
    "x_pos"   : 1,
    "y_pos"   : 116,
    "enable"  : 1
  }
}
 */

#define LILYGO_T5_V213                                  // Modèle d'afficheur utilisé
#include "LilygoT5_213.h"                               // Défini les I/O de la carte afficheur 
#include <GxEPD2_BW.h>                                  // Librairie pour version monochrome
#include <ArduinoJson.h>                                // Librairie pour la lecture du fichier de configuration sur la carte SD (format JSON)
#include <SD.h>                                         // Librairie de gestion de la carte SD

/***************************************************************************************************************************************************************/
// Macro configuration driver / pinout hardware spécifique à la carte LilyGO T5

// Driver pour mon e.paper LilyGO T5 (version de 2017 - IL3895 pour le contrôleur - F5FDR le 01/02/2023)
//GxEPD2_BW<GxEPD2_213, GxEPD2_213::HEIGHT> display(GxEPD2_213(/*CS=*/ EPD_CS, /*DC=*/ EPD_DC, /*RST=*/ EPD_RSET, /*BUSY=*/ EPD_BUSY));         // DEPG0213BN 104x212, IL3895, TTGO T5 V2.3.1

// Driver pour mon e.paper LilyGO T5 (version de 2023 - SSD1680 pour le contrôleur - F5FDR le 10/02/2023)
GxEPD2_BW<GxEPD2_213_BN, GxEPD2_213_BN::HEIGHT> display(GxEPD2_213_BN(/*CS=5*/ EPD_CS, /*DC=*/ EPD_DC, /*RST=*/ EPD_RSET, /*BUSY=*/ EPD_BUSY)); // DEPG0213BN 128x250, SSD1680, TTGO T5 V2.4.1, V2.3.1
/***************************************************************************************************************************************************************/

#define __SERIAL_DEBUG__    true                      // Mettre à true si l'on souhaite suivre les phases de fonctionnement sur la liaison série à 115200 bds

// Charge les librairies locales propres à l'application badge

#include "LogoREF-50x71.h"                            // Ajoute le logo du REF 
#include "defaultData.h"                              // Fichier contenant les données par défaut qu'il est possible de modifier
#include "badgeV2.h"                                  // Fonctions de gestion de l'affichage. Réservé aux initiés à la programmation

void setup()
{
  // Initialise le port série pour les phases de DEBUG
  display.init();                                     // Initialisation de l'afficheur
  display.setRotation(-1);                            // Sélection du sens d'affichage (-1 / 0 / 1 / 2)
                                                      // -1 = Horizontal: Connecteur USB vers le bas
                                                      // 0  = Vertical:   Connecteur USB vers la droite (affichage par défaut) 
                                                      // 1  = Horizontal: Connecteur USB vers le haut
                                                      // 2  = Vertical:   Connecteur USB vers la gauche
                                                      // Note : Les positions actuelles des textes sont définie pour le sens "-1"
                                                      //        Tout changement nécessitera de redéfinir les positions X & Y
                                                      //        de tous les textes ainsi que le choix des tailles des polices 
                                                      //        de caractères.

  display.setTextColor(GxEPD_BLACK);                  // Défini l'encre et le fond d'écran par défaut (Texte en noir sur fond blanc - deux couleurs possible seulement)
  display.fillScreen(GxEPD_WHITE);                    // Sélectionne le fond d'écran en blanc
  display.display(true);                              // Mise à jour de l'afficheur

  // Initialisation du port série de l'afficheur
#if (__SERIAL_DEBUG__ == true)
  Serial.begin(115200);                               // Liaison à 115200 bauds
  delay(1000);                                        // Délai d'une seconde
  Serial.println();                                   // Ligne blanche
  Serial.println("E.Paper Electronic Badge V2");
  Serial.println("F5FDR - 02-2023\n");
  Serial.println("Init. E-Paper: Ok");
#endif

  // Charge les données par défaut avant de lire la carte SD
  loadDefaultValue(&Config);
  
  // Initialise le driver de gestion de la carte SD
  bSdOk = setupSDCard();                              // Teste si une carte SD est installée
#if (__SERIAL_DEBUG__ == true)
  String sizeString = "SD:" + String(SD.cardSize() / 1024.0 / 1024.0 / 1024.0) + " Go"; // Info sur la taille de la carte
  Serial.println(bSdOk ? sizeString : "SD:None");
#endif

  // Charge le fichier de configuration s'il existe sur la carte SD
  if (bSdOk) {
    checkUserJsonFile(configFile, &bSdOk);            // Met a jour le flag bSdOk si erreur trouvée
  }

  // Affiche tous les textes
  displayCallSign(&Config);                           // Affichage des textes du badge

  display.powerOff();
  display.hibernate();
}

/* Boucle principale non utilisée dans cette version.
 * C'est dans cette boucle que l'on peut ajouter du code si l'on souhaite
 * créer des animations sur l'afficheur ou toutes autres applications
 * spécifiques.
 */
void loop() {};                                       // Non utilisée dans cette version
